@@ -33,7 +33,7 @@ static const struct vpd_item {
 } vpd_items[] = {
   { 'C','P', F_BINARY,	"Extended capability" },
   { 'E','C', F_TEXT,	"Engineering changes" },
-  { 'M','N', F_BINARY,	"Manufacture ID" },
+  { 'M','N', F_TEXT,	"Manufacture ID" },
   { 'P','N', F_TEXT,	"Part number" },
   { 'R','V', F_RESVD,	"Reserved" },
   { 'R','W', F_RDWR,	"Read-write area" },
@@ -41,6 +41,13 @@ static const struct vpd_item {
   { 'Y','A', F_TEXT,	"Asset tag" },
   { 'V', 0 , F_TEXT,	"Vendor specific" },
   { 'Y', 0 , F_TEXT,	"System specific" },
+  /* Non-standard extensions */
+  { 'C','C', F_TEXT,	"CCIN" },
+  { 'F','C', F_TEXT,	"Feature code" },
+  { 'F','N', F_TEXT,	"FRU" },
+  { 'N','A', F_TEXT,	"Network address" },
+  { 'R','M', F_TEXT,	"Firmware version" },
+  { 'Z', 0 , F_TEXT,	"Product specific" },
   {  0,  0 , F_BINARY,	"Unknown" }
 };
 
